@@ -137,6 +137,8 @@ gssdp_client_constructed (GObject *object)
                          request_socket_source_cb,
                          client,
                          NULL);
+        } else {
+          goto errors;
         }
 
         client->priv->multicast_socket =
@@ -150,12 +152,13 @@ gssdp_client_constructed (GObject *object)
                          NULL);
         }
 
+ errors:
         if (!client->priv->request_socket || !client->priv->multicast_socket) {
                 if (client->priv->error)
                         g_set_error_literal (client->priv->error,
                                              GSSDP_ERROR,
                                              GSSDP_ERROR_FAILED,
-                                             strerror (errno));
+                                             g_strerror (errno));
 
                 return;
         }
@@ -845,6 +848,8 @@ get_host_ip (char **interface)
                         continue;
                 else if (!(ifa->ifa_flags & IFF_UP))
                         continue;
+		else if ((ifa->ifa_flags & IFF_POINTOPOINT))
+			continue;
 
                 /* Loopback and IPv6 interfaces go at the bottom on the list */
                 if (ifa->ifa_flags & IFF_LOOPBACK ||
@@ -915,7 +920,7 @@ init_network_info (GSSDPClient *client)
                 if (client->priv->error)
                         g_set_error (client->priv->error,
                                      GSSDP_ERROR,
-                                     GSSDP_ERROR_FAILED,
+                                     GSSDP_ERROR_NO_IP_ADDRESS,
                                      "Failed to find IP of interface %s",
                                      client->priv->interface);
 
